
from logging import Logger
from logging import getLogger

from click import secho

from py2appsigner.CommandBasic import CommandBasic
from py2appsigner.environment.Environment import Environment


class ApplicationStaple(CommandBasic):

    def __init__(self, environment: Environment):

        super().__init__(environment=environment)
        self.logger: Logger = getLogger(__name__)

    def execute(self):
        # echo "${txReverse}Start stapling: ${FULL_APP_NAME}${txReset}"
        # xcrun stapler staple ${FULL_APP_NAME}
        secho('Call Apple for notary service', reverse=True)

        stapleIt: str = f'xcrun stapler staple {self._applicationName}'

        self._runCommand(command=stapleIt)




import logging
import logging.config

from json import load as jsonLoad

from importlib.resources import files
from importlib.abc import Traversable

from click import command
from click import group
from click import pass_context
from click import pass_obj
from click import secho
from click import clear
from click import version_option
from click import option

from py2appsigner import __version__ as version

from py2appsigner.ApplicationNotarize import ApplicationNotarize
from py2appsigner.ApplicationSign import ApplicationSign
from py2appsigner.ApplicationStaple import ApplicationStaple
from py2appsigner.ApplicationVerify import ApplicationVerify
from py2appsigner.Notary import Notary
from py2appsigner.environment.BasicEnvironment import BasicEnvironment

from py2appsigner.environment.Environment import Environment
from py2appsigner.ZipSign import ZipSign
from py2appsigner.environment.NotaryEnvironment import NotaryEnvironment


RESOURCES_PACKAGE_NAME:       str = 'py2appsigner.resources'
JSON_LOGGING_CONFIG_FILENAME: str = "loggingConfiguration.json"

VERBOSE_OPTION_HELP: str = 'Include this option to instruct the command to echo the underlying CLI output'

"""
Put in type ignore because of strange error on that appeared on 8.1.4

buildlackey/Commands.py:80: error: Argument 1 has incompatible type "Callable[[], Any]"; expected <nothing>  [arg-type]
    @command
"""


def setUpLogging():
    """
    """
    traversable: Traversable = files(RESOURCES_PACKAGE_NAME) / JSON_LOGGING_CONFIG_FILENAME

    loggingConfigFilename: str = str(traversable)

    with open(loggingConfigFilename, 'r') as loggingConfigurationFile:
        configurationDictionary = jsonLoad(loggingConfigurationFile)

    logging.config.dictConfig(configurationDictionary)
    logging.logProcesses = False
    logging.logThreads = False


@group
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--application-name',  '-a', required=True,  help='The application name that py2app built')
@option('--projects-base',     '-b', required=False, help='Projects base, overrides environment variable')
@option('--project-directory', '-d', required=False, help='Project directory, overrides environment variable')
@option('--python-version',    '-p', required=True,  help='Identify the python version')
@option('--identity',          '-i', required=False, help='Code signing identity')
@option('--verbose',           '-v', required=False, is_flag=True, help=VERBOSE_OPTION_HELP)
@pass_context
def py2appSign(ctx, python_version: str, application_name: str, projects_base: str = '', project_directory: str = '', identity: str = '', verbose: bool = False):
    """
    Specify a python version that the py2app application is using.
    \b

    Specify the application name created by py2app
    \b

    The environment variable for projects base is 'PROJECTS_BASE'.  This is a fully qualified
    directory name.
    \b

    The environment variable for project directory is 'PROJECT'.  This is just the
    simple project directory name.

    identity -- For code signing, a digital identity must be stored in a keychain that is on the calling user's keychain search list.
    if not specified then the value must be set in the 'IDENTITY' environment variable
    \b
    """
    setUpLogging()

    environment: Environment     = Environment(pythonVersion=python_version,
                                               applicationName=application_name,
                                               projectsBase=projects_base,
                                               projectDirectory=project_directory,
                                               identity=identity,
                                               verbose=verbose)

    ctx.obj = environment


@py2appSign.command()
@pass_obj
def zipSign(environment: Environment):

    zipsign: ZipSign = ZipSign(environment=environment)
    zipsign.execute()


@py2appSign.command()
@option('--fix-lib',      '-l', required=False, is_flag=True, help='Fix broken library')
@option('--fix-sym-link', '-s', required=False, is_flag=True, help='Fix invalid symbolic link')
@pass_obj
def appSign(environment: Environment, fix_lib: bool = False, fix_sym_link: bool = False):
    """
    fix-lib gets the following dynamic library from Homebrew;  And copies it into the
    Python virtual environment;  Works only on Apple Silicon OS X
    and with Homebrew installed

    See: https://stackoverflow.com/questions/62095338/py2app-fails-macos-signing-on-liblzma-5-dylib

    On Intel OS X

    /usr/local/Cellar/xz/5.2.5/lib/liblzma.5.dylib

    Apple Silicon

    /opt/homebrew/opt/xz/lib/liblzma.5.dylib

    -fix-sym-link removes the following symbolic link from the application binary before signing

     <application>.app/Contents/Resources/lib/python<python version>/site.pyo

     Leaving this file in place with a signed and notarized application causes it to
     fail the appVerify phase and renders the binary unusable

    """
    applicationSign: ApplicationSign = ApplicationSign(environment=environment, fixLib=fix_lib, fixSymLink=fix_sym_link)
    applicationSign.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--application-name',  '-a', required=True,  help='The application name that py2app built')
@option('--projects-base',     '-b', required=False, help='Projects base, overrides environment variable')
@option('--project-directory', '-d', required=False, help='Project directory, overrides environment variable')
@option('--verbose',           '-v', required=False, is_flag=True, help=VERBOSE_OPTION_HELP)
def appNotarize(application_name: str, projects_base: str = '', project_directory: str = '', verbose: bool = False):
    """
    Specify the application name created by py2app
    \b

    The environment variable for projects base is 'PROJECTS_BASE'.  This is a fully qualified
    directory name.
    \b

    The environment variable for project directory is 'PROJECT'.  This is just the
    simple project directory name.

    Assumes the developer stored application specific ID with the name 'APP_PASSWORD'

    """
    environment: BasicEnvironment = BasicEnvironment(applicationName=application_name, projectsBase=projects_base, projectDirectory=project_directory, verbose=verbose)

    applicationNotarize: ApplicationNotarize = ApplicationNotarize(environment=environment)
    applicationNotarize.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--application-name',  '-a', required=True,  help='The application name that py2app built')
@option('--projects-base',     '-b', required=False, help='Projects base, overrides environment variable')
@option('--project-directory', '-d', required=False, help='Project directory, overrides environment variable')
@option('--verbose',           '-v', required=False, is_flag=True, help=VERBOSE_OPTION_HELP)
def appStaple(application_name: str, projects_base: str = '', project_directory: str = '', verbose: bool = False):

    environment: Environment     = Environment(pythonVersion='',                    # Not Needed
                                               applicationName=application_name,
                                               projectsBase=projects_base,
                                               projectDirectory=project_directory,
                                               identity='',                         # Not needed
                                               verbose=verbose)

    applicationStaple: ApplicationStaple = ApplicationStaple(environment=environment)
    applicationStaple.execute()


@command()
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--application-name',  '-a', required=True,  help='The application name that py2app built')
@option('--projects-base',     '-b', required=False, help='Projects base, overrides environment variable')
@option('--project-directory', '-d', required=False, help='Project directory, overrides environment variable')
@option('--verbose',           '-v', required=False, is_flag=True, help=VERBOSE_OPTION_HELP)
def appVerify(application_name: str, projects_base: str = '', project_directory: str = '', verbose: bool = False):

    environment: Environment     = Environment(pythonVersion='',                    # Not Needed
                                               applicationName=application_name,
                                               projectsBase=projects_base,
                                               projectDirectory=project_directory,
                                               identity='',                         # Not needed
                                               verbose=verbose)

    applicationVerify: ApplicationVerify = ApplicationVerify(environment=environment)
    applicationVerify.execute()


@group
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--keychain-profile', '-p', required=False, help='Keychain profile name storing Notary Tool Application Id')
@pass_context
def notaryTool(ctx, keychain_profile: str):
    """
    Use this command to request information about a specific submission or a history of all
    your submissions.

    The default keychain profile name is 'NOTARY_TOOL_APP_ID'
    """
    notaryEnvironment: NotaryEnvironment = NotaryEnvironment()
    if keychain_profile is not None:
        notaryEnvironment.keyChainProfile = keychain_profile
    ctx.obj = notaryEnvironment


@notaryTool.command()
@pass_obj
def history(notaryEnvironment: NotaryEnvironment):
    """
    """
    notary: Notary = Notary(notaryEnvironment=notaryEnvironment)
    notary.history()


@notaryTool.command()
@option('--submission-id', '-i', required=True, help='Submission ID returned from a previous invocation of `appNotarize`')
@pass_obj
def information(notaryEnvironment: NotaryEnvironment, submission_id: str):
    """
    """
    notary: Notary = Notary(notaryEnvironment=notaryEnvironment)
    notary.information(submissionId=submission_id)


@command()
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
def py2appsigner():
    clear()
    secho('Example Commands:')
    secho('     py2appSign -p 3.11 -d pyut -a Pyut  --verbose zipsign')
    secho('     py2appSign -p 3.11 -d pyut -a Pyut  --verbose appsign')
    secho('')
    secho('     appNotarize -d pyut -a Pyut --verbose')
    secho('     appStaple   -d pyut -a Pyut --verbose')
    secho('     appVerify   -d pyut -a Pyut')
    secho('')
    secho('     notaryTool history')
    secho('     notaryTool -p NOTARY_TOOL_APP_ID history')
    secho('     notaryTool information -i <submission id>')
    secho('     notaryTool -p NOTARY_TOOL_APP_ID information -i <submission id>')


if __name__ == '__main__':
    # noinspection SpellCheckingInspection
    """
    py2appSign(['--python-version', '3.10', '-d', 'pyut', '--application-name', 'pyut', 'zipsign'])
    py2appSign(['--python-version', '3.10', '-d', 'pyut', '--application-name', 'pyut', 'appsign'])
    appNotarize(['-d', 'pyut', '--application-name', 'pyut', '--verbose'])
    appStaple(['-d', 'pyut', '--application-name', 'pyut', '--verbose'])
    notaryTool(['information', '-i', '5f57fc1e-23d3-42ab-b0ad-ec1d2635c4ad'])
    notaryTool(['--keychain-profile', 'NOTARY_TOOL_APP_ID', 'history'])
    appVerify(['-a', 'Pyut', '-d', 'pyut', '--verbose'])
    """
    py2appSign(['-p', '3.11', '-d', 'pyut', '-a', 'pyut', '--verbose', 'appsign', '--fix-sym-link'])



from typing import List

from logging import Logger
from logging import getLogger

from pathlib import Path

from click import secho

from tqdm import tqdm

from py2appsigner.CommandBasic import CODE_SIGN_OPTIONS_QUIET
from py2appsigner.CommandBasic import CODE_SIGN_OPTIONS_VERBOSE
from py2appsigner.CommandBasic import CODE_SIGN_TOOL
from py2appsigner.CommandBasic import COPY_OPTIONS_VERBOSE
from py2appsigner.CommandBasic import COPY_OPTIONS_QUIET
from py2appsigner.CommandBasic import REMOVE_OPTIONS_QUIET
from py2appsigner.CommandBasic import REMOVE_OPTIONS_VERBOSE

from py2appsigner.CommandExtended import CommandExtended

from py2appsigner.Common import PROGRESS_BAR_UNITS

from py2appsigner.environment.Environment import Environment


# noinspection SpellCheckingInspection
"""
# https://stackoverflow.com/questions/62095338/py2app-fails-macos-signing-on-liblzma-5-dylib
#
# export GOOD_LIB='/usr/local/Cellar/xz/5.2.5/lib/liblzma.5.dylib'
# if you are on the new Apple Silicon homebrew is now here:
"""

# noinspection SpellCheckingInspection
GOOD_LIB: str = '/opt/homebrew/opt/xz/lib/liblzma.5.dylib'

CLEAN_UP_CRUD: List[str] = [
    'Contents/Resources/lib/python3.10/todoist/.DS_Store',
    'Contents/Resources/lib/python3.10/numpy/f2py/tests/src/assumed_shape/.f2py_f2cmap',
    'Contents/Resources/lib/python3.10/site.pyo'
]

SHARED_OBJECT_LIBRARY_WILDCARD:       str = '*.so'
MACH_OBJECT_DYNAMIC_LIBRARY_WILDCARD: str = '*.dylib'

PRE_FRAMEWORK_PATH: str = '/Contents/Frameworks/Python.framework/Versions'


class ApplicationSign(CommandExtended):

    def __init__(self, environment: Environment, fixLib: bool, fixSymLink: bool):

        super().__init__(environment=environment)
        self.logger: Logger = getLogger(__name__)

        self._fixLib:     bool = fixLib
        self._fixSymLink: bool = fixSymLink

        self._codeSignCommand: str = self._constructCodeSignCommand()

    def execute(self):

        self._fixLibrary()
        self._fixSymbolicLink()
        self._cleanupCrud()
        self._signFrameworks()
        self._signLibraries()
        self._signPythonApp()
        self._signApplication()

    def _fixLibrary(self):

        if self._fixLib is True:
            directoryToOverWrite: str = f'{self._applicationName}/Contents/Frameworks'
            options:              str = self._getToolOptions(verboseOptions=COPY_OPTIONS_VERBOSE, quietOptions=COPY_OPTIONS_QUIET)
            overwrite:            str = f'cp {options} {GOOD_LIB} {directoryToOverWrite}'
            self._runCommand(overwrite)
            secho(f'Fixed library: {overwrite}')

    def _fixSymbolicLink(self):
        if self._fixSymLink is True:
            linkToRemove: str = f'{self._applicationName}/Contents/Resources/lib/python{self._pythonVersion}/site.pyo '
            remove:       str = f'rm {REMOVE_OPTIONS_VERBOSE} {linkToRemove}'
            secho(remove)
            self._runCommand(remove)

    def _cleanupCrud(self):
        """
        Ugh code signing will be the death of me
        Either invalid links or something code signing or verifying complains about
        """
        removeOptions: str = self._getToolOptions(verboseOptions=REMOVE_OPTIONS_VERBOSE, quietOptions=REMOVE_OPTIONS_QUIET)
        for partialCrud in CLEAN_UP_CRUD:
            fullCrud: Path = Path(self._applicationName) / partialCrud

            removeIt: str = f'rm {removeOptions} {fullCrud}'

            self._runCommand(removeIt)

    def _signLibraries(self):
        # noinspection SpellCheckingInspection
        """
        find "${FULL_APP_NAME}" -iname '*.so' -or -iname '*.dylib' |
            while read libfile; do
                codesign --sign "${IDENTITY}" ${OPTIONS} "${libfile}" >> ${LOGFILE} 2>&1 ;
            done;
        """
        p: Path = Path(self._applicationName)

        secho(f'Sign Libraries - {SHARED_OBJECT_LIBRARY_WILDCARD}')
        soLibs: List[Path] = sorted(p.rglob(f'{SHARED_OBJECT_LIBRARY_WILDCARD}'))

        if self.verbose is True:
            self._verboseSignSOLibraries(soLibs)
        else:
            self._progressBarSignSOLibraries(soLibs)

        secho(f'Sign Libraries - {MACH_OBJECT_DYNAMIC_LIBRARY_WILDCARD}')
        dyLibs: List[Path] = sorted(p.rglob(f'{MACH_OBJECT_DYNAMIC_LIBRARY_WILDCARD}'))

        if self.verbose is True:
            self._verboseSignDynamicLibraries(dyLibs)
        else:
            self._progressBarSignDynamicLibraries(dyLibs)

    def _signFrameworks(self):
        """
         Assumes Xcode 13 is installed
        """
        #
        # codesign --sign "${IDENTITY}" ${OPTIONS} "${FULL_APP_NAME}/Contents/Frameworks/Python.framework/Versions/3.10/Python"
        #
        secho('Sign Framework')

        framework:     str = (
            f'{self._applicationName}/{PRE_FRAMEWORK_PATH}/{self._pythonVersion}/Python'
        )
        signFramework: str = f'{self._codeSignCommand} {framework}'
        self._runCommand(signFramework)

    def _signPythonApp(self):
        # codesign --sign "${IDENTITY}" ${OPTIONS} "${FULL_APP_NAME}/Contents/macOS/python"
        pythonApp:  str = f'{self._applicationName}/Contents/MacOS/python'
        signPython: str = f'{self._codeSignCommand} {pythonApp}'

        self._runCommand(signPython)

    def _signApplication(self):
        # codesign --sign "${IDENTITY}" ${OPTIONS} "${FULL_APP_NAME}/Contents/MacOS/${APPLICATION_NAME}"
        application:     str = f'{self._applicationName}/Contents/MacOS/{self._extendedEnvironment.applicationName}'
        signApplication: str = f'{self._codeSignCommand} {application}'

        self._runCommand(signApplication)

    def _constructCodeSignCommand(self) -> str:
        """
        It is up to the caller to append it item that needs to be signed to code sign CLI

        codesign --sign "${IDENTITY}" ${OPTIONS} <CALLER APPENDS TARGET>

        Returns:  The code CLI to invoke with all options correctly set;
        """
        options:  str = self._getToolOptions(verboseOptions=CODE_SIGN_OPTIONS_VERBOSE, quietOptions=CODE_SIGN_OPTIONS_QUIET)
        identity: str = self._extendedEnvironment.identity

        codeSignCommand: str = f'{CODE_SIGN_TOOL} --sign "{identity}" {options}'

        return codeSignCommand

    def _verboseSignSOLibraries(self, soLibs: List[Path]):
        for lib in soLibs:
            signSo: str = f'{self._codeSignCommand} {lib}'
            self._runCommand(signSo)

    def _progressBarSignSOLibraries(self, soLibs: List[Path]):
        pbar: tqdm = tqdm(soLibs, unit=PROGRESS_BAR_UNITS)
        for lib in pbar:
            path: Path = Path(lib)
            pbar.set_description(f'Processing {path.name}')
            signSo: str = f'{self._codeSignCommand} {lib}'
            self._runCommand(signSo)

    def _verboseSignDynamicLibraries(self, dyLibs: List[Path]):
        for lib in dyLibs:
            signDyLibs: str = f'{self._codeSignCommand} {lib}'
            self._runCommand(signDyLibs)

    def _progressBarSignDynamicLibraries(self, dyLibs: List[Path]):
        pbar: tqdm = tqdm(dyLibs, unit=PROGRESS_BAR_UNITS)
        for dylib in pbar:
            path: Path = Path(dylib)
            pbar.set_description(path.name)
            signDyLibs: str = f'{self._codeSignCommand} {dylib}'
            self._runCommand(signDyLibs)




from logging import Logger
from logging import getLogger

from click import secho

from py2appsigner.CommandBasic import CommandBasic
from py2appsigner.environment.Environment import Environment

# noinspection SpellCheckingInspection
SECURITY_ASSESSMENT_COMMAND: str = '/usr/sbin/spctl'


class ApplicationVerify(CommandBasic):

    def __init__(self, environment: Environment):

        super().__init__(environment=environment)

        self.logger: Logger = getLogger(__name__)

    def execute(self):

        secho('Verify signature', reverse=True)
        verify: str = f'{SECURITY_ASSESSMENT_COMMAND} -vvvv --assess --type exec {self._applicationName}'

        self._runCommand(verify)




from logging import Logger
from logging import getLogger

from click import clear
from click import secho

from py2appsigner.Common import DEFAULT_NOTARY_TOOL_KEYCHAIN_PROFILE_NAME
from py2appsigner.environment.BasicEnvironment import BasicEnvironment

from py2appsigner.CommandBasic import BUILD_DIR
from py2appsigner.CommandBasic import CommandBasic
from py2appsigner.CommandBasic import DITTO_TOOL
from py2appsigner.CommandBasic import REMOVE_OPTIONS_QUIET
from py2appsigner.CommandBasic import REMOVE_OPTIONS_VERBOSE
from py2appsigner.CommandBasic import ZIP_SUFFIX


class ApplicationNotarize(CommandBasic):

    def __init__(self, environment: BasicEnvironment, keyChainProfileName: str = DEFAULT_NOTARY_TOOL_KEYCHAIN_PROFILE_NAME):
        super().__init__(environment=environment)

        self._keyChainProfileName: str    = keyChainProfileName
        self.logger:               Logger = getLogger(__name__)

    def execute(self):
        #
        #  assumes Xcode 13 is installed
        #  assumes you added an entry APP_PASSWORD to your keychain
        #
        clear()
        zipFile:       str = f'{self._fullPath}{BUILD_DIR}{self._basicEnvironment.applicationName}.{ZIP_SUFFIX}'

        self._cleanupOldZipFile(zipFile=zipFile)
        self._createNewZipFile(zipFile=zipFile)
        self._notarizeIt(zipFile=zipFile)
        self._removeZipFile(zipFile=zipFile, message='Success! Let`s be hygienic')

    def _cleanupOldZipFile(self, zipFile: str):
        # echo "${txReverse}Clean up in case of restart on failure${txReset}"
        # rm - rfv "${ZIP_PATH}"
        # echo    "${txReverse}Clean up in case of restart on failure${txReset}"

        self._removeZipFile(zipFile=zipFile, message='Clean up in case of restart on failure')

    def _createNewZipFile(self, zipFile: str):
        # echo "${txReverse}Create a ZIP archive suitable for notarization${txReset}"
        # /usr/bin/ditto - c - k - -keepParent "${APP_PATH}" "${ZIP_PATH}"

        secho('Create a ZIP archive suitable for notarization', reverse=True)

        zipIt: str = f'{DITTO_TOOL} -c -k --keepParent {self._applicationName} {zipFile}'

        self._runCommand(zipIt)

    def _removeZipFile(self, zipFile: str, message: str):
        secho(message=message, reverse=True)

        removeOptions: str = self._getToolOptions(verboseOptions=REMOVE_OPTIONS_VERBOSE, quietOptions=REMOVE_OPTIONS_QUIET)
        removeIt:      str = f'rm {removeOptions} {zipFile}'

        self._runCommand(removeIt)

    def _notarizeIt(self, zipFile: str):
        secho('Call Apple for notary service', reverse=True)

        # noinspection SpellCheckingInspection
        notarizeIt: str = f'xcrun notarytool submit {zipFile} --keychain-profile "{self._keyChainProfileName}" --wait'

        self._runCommand(notarizeIt)




from dataclasses import dataclass

from py2appsigner.Common import DEFAULT_NOTARY_TOOL_KEYCHAIN_PROFILE_NAME


@dataclass
class NotaryEnvironment:

    keyChainProfile: str  = DEFAULT_NOTARY_TOOL_KEYCHAIN_PROFILE_NAME




from logging import Logger
from logging import getLogger
from subprocess import CalledProcessError

from sys import stdout

from subprocess import run as subProcessRun
from subprocess import check_call as subProcessCheckCall
from subprocess import STDOUT

from click import ClickException
from click import secho


class BaseCommand:
    def __init__(self, verbose: bool = True):

        self._verbose: bool   = verbose
        self.logger:   Logger = getLogger(__name__)

    @property
    def verbose(self) -> bool:
        return self._verbose

    def _getToolOptions(self, verboseOptions: str, quietOptions: str) -> str:

        if self._verbose is True:
            return verboseOptions
        else:
            return quietOptions

    def _runCommand(self,  command: str):

        try:
            if self._verbose is True:
                secho(self._execute(command=command))
            else:
                subProcessRun([command], shell=True, capture_output=True, text=True, check=True)
        except CalledProcessError as cpe:
            raise ClickException(message=f'Fail {cpe.cmd}')

    def _execute(self, command):
        subProcessCheckCall(command, shell=True, stdout=stdout, stderr=STDOUT)




from logging import Logger
from logging import getLogger

from subprocess import CompletedProcess
from subprocess import run as subProcessRun
from subprocess import CalledProcessError

from click import secho

from py2appsigner.BaseCommand import BaseCommand
from py2appsigner.environment.NotaryEnvironment import NotaryEnvironment


# noinspection SpellCheckingInspection
PARTIAL_NOTARY_TOOL_CLI:     str = ' xcrun notarytool '
PARAMETER_KEY_CHAIN_PROFILE: str = ' --keychain-profile '
NOTARY_HISTORY_FILENAME:     str = 'notaryHistory.log'


class Notary(BaseCommand):

    def __init__(self, notaryEnvironment: NotaryEnvironment):

        super().__init__()

        self.logger: Logger = getLogger(__name__)

        self._notaryEnvironment: NotaryEnvironment = notaryEnvironment

    def history(self):
        # noinspection SpellCheckingInspection
        """
        xcrun notarytool history --keychain-profile "NOTARY_TOOL_APP_ID"  >> notaryhistory.log
        """
        historyRequest: str = (
            f'{PARTIAL_NOTARY_TOOL_CLI} history {PARAMETER_KEY_CHAIN_PROFILE} {self._notaryEnvironment.keyChainProfile}'
        )
        # Don't use base command method;  I want to capture the output and write it to a file
        completedProcess: CompletedProcess = subProcessRun([historyRequest], shell=True, capture_output=True, text=True, check=True)
        if completedProcess.returncode == 0:
            with open(NOTARY_HISTORY_FILENAME, 'w') as fd:
                fd.write(completedProcess.stdout)
            secho(f'See: {NOTARY_HISTORY_FILENAME}')

    def information(self, submissionId: str):
        # noinspection SpellCheckingInspection
        """
        xcrun notarytool log $notarizationId --keychain-profile "NOTARY_TOOL_APP_ID" "notary-${notarizationId}.log"
        """
        outputFile: str = f'notary-{submissionId}.log'

        # noinspection SpellCheckingInspection
        logRequest: str = (
            f'{PARTIAL_NOTARY_TOOL_CLI} log {submissionId} '
            f'{PARAMETER_KEY_CHAIN_PROFILE} "{self._notaryEnvironment.keyChainProfile}" '
            f'"{outputFile}"'
        )
        try:
            self._runCommand(command=logRequest)
            secho(f'Output is in {outputFile}')
        except CalledProcessError as cpe:
            secho(f'{cpe.stderr}')




from dataclasses import dataclass
from logging import Logger
from logging import getLogger

from os import environ as osEnvironment

from click import ClickException


@dataclass
class BasicEnvironment:

    ENV_PROJECTS_BASE: str = 'PROJECTS_BASE'
    ENV_PROJECT:       str = 'PROJECT'

    applicationName:  str = ''
    projectsBase:     str = ''
    projectDirectory: str = ''
    verbose:          bool = False

    def __init__(self, applicationName: str = '', projectsBase: str = '', projectDirectory: str = '', verbose: bool = False):
        """
        Arguments for the command line always override the environment variables

        Args:
            applicationName:
            projectsBase:     Base directory for python projects.
            projectDirectory: The project directory name
            verbose:          'True' if we echo command names and status results from their execution
        """
        self.logger: Logger = getLogger(__name__)

        self.applicationName  = applicationName
        self.projectsBase     = projectsBase
        self.projectDirectory = projectDirectory
        self.verbose          = verbose

        if self.validProjectsBase is False:
            try:
                self.projectsBase = osEnvironment[BasicEnvironment.ENV_PROJECTS_BASE]
            except KeyError:
                raise ClickException(message='I do not know the base directory name of your Python projects')

        if self.validProjectDirectory is False:
            try:
                self.projectDirectory = osEnvironment[BasicEnvironment.ENV_PROJECT]
            except KeyError:
                raise ClickException(message='I do not know the name of the project directory')

    @property
    def validProjectsBase(self) -> bool:
        if self.projectsBase == '' or self.projectsBase is None:
            return False
        else:
            return True

    @property
    def validProjectDirectory(self) -> bool:
        if self.projectDirectory == '' or self.projectDirectory is None:
            return False
        else:
            return True




from logging import Logger
from logging import getLogger

from dataclasses import dataclass

from os import environ as osEnvironment

from click import ClickException

from py2appsigner.environment.BasicEnvironment import BasicEnvironment


@dataclass
class Environment(BasicEnvironment):
    """
    """
    IDENTITY:          str = 'IDENTITY'

    pythonVersion:    str = ''
    identity:         str  = ''

    def __init__(self, pythonVersion: str, applicationName: str, projectsBase: str = '', projectDirectory: str = '', identity: str = '', verbose: bool = False):
        """
        Arguments for the command line always override the environment variables

        Args:
            pythonVersion:
            applicationName:
            projectsBase:     Base directory for python projects.
            projectDirectory: The project directory name
            verbose:          'True' if we echo command names and status results from their execution
        """
        super().__init__(applicationName=applicationName, projectsBase=projectsBase, projectDirectory=projectDirectory, verbose=verbose)
        self.logger: Logger = getLogger(__name__)

        self.pythonVersion    = pythonVersion
        self.identity         = identity

        if self.identity == '' or self.identity is None:
            try:
                self.identity = osEnvironment[Environment.IDENTITY]
            except KeyError:
                raise ClickException(message='You must provide the IDENTITY environment variable')




from logging import Logger
from logging import getLogger

from abc import abstractmethod

from py2appsigner.CommandBasic import CommandBasic
from py2appsigner.environment.Environment import Environment


class CommandExtended(CommandBasic):
    def __init__(self, environment: Environment):

        super().__init__(environment=environment)
        self.logger: Logger = getLogger(__name__)

        self._extendedEnvironment: Environment = environment
        self._pythonVersion:       str         = self._extendedEnvironment.pythonVersion
        self._flatPythonVersion:   str         = self._removeDecimalSeparator(self._extendedEnvironment.pythonVersion)

    @abstractmethod
    def execute(self):
        pass

    def _removeDecimalSeparator(self, pythonVersion: str):
        return pythonVersion.replace('.', '')




from logging import Logger
from logging import getLogger

from os import sep as osSep

from pathlib import Path

from tqdm import tqdm

from click import secho

from py2appsigner.CommandBasic import CODE_SIGN_OPTIONS_QUIET
from py2appsigner.CommandBasic import CODE_SIGN_OPTIONS_VERBOSE
from py2appsigner.CommandBasic import CODE_SIGN_TOOL

from py2appsigner.CommandBasic import COPY_OPTIONS_VERBOSE
from py2appsigner.CommandBasic import COPY_OPTIONS_QUIET
from py2appsigner.CommandBasic import REMOVE_OPTIONS_QUIET
from py2appsigner.CommandBasic import REMOVE_OPTIONS_VERBOSE
from py2appsigner.CommandExtended import CommandExtended

from py2appsigner.Common import PROGRESS_BAR_UNITS

from py2appsigner.environment.Environment import Environment

ZIP_DIRECTORY_SNIPPET: str = '/Contents/Resources/lib'
TMP_DIR_PATH:          str = '/tmp'

DITTO_TOOL:                str = '/usr/bin/ditto'

DITTO_EXTRACT_OPTIONS_VERBOSE: str = '-x -k -v'
DITTO_EXTRACT_OPTIONS_QUIET:   str = '-x -k'
DITTO_CREATE_OPTIONS_VERBOSE:  str = '-c -k -v'
DITTO_CREATE_OPTIONS_QUIET:    str = '-c -k'


class ZipSign(CommandExtended):

    def __init__(self, environment: Environment):

        super().__init__(environment=environment)

        self.logger: Logger = getLogger(__name__)

    def execute(self):
        self.logger.debug(f'{self._extendedEnvironment.pythonVersion=} {self._extendedEnvironment.applicationName=}')

        # fullPath:        str = f'{self._environment.projectsBase}/{self._environment.projectDirectory}'
        # pythonVersion:   str = self._removeDecimalSeparator(self._environment.pythonVersion)
        # applicationName: str = f'{fullPath}{BUILD_DIR}{self._environment.applicationName}.app'

        originalZipDir: str = f'{self._applicationName}{ZIP_DIRECTORY_SNIPPET}'
        unzipDir:       str = f'{TMP_DIR_PATH}/python{self._flatPythonVersion}'
        zipName:        str = f'python{self._flatPythonVersion}.zip'

        self._cleanupTemporaryDirectory(unzipDir=unzipDir, zipName=zipName)
        self._getUnsignedZipCopy(originalZipDir=originalZipDir, zipName=zipName)
        self._unzipCopy(unzipDir=unzipDir, zipName=zipName)
        self._signLibs(unzipDir=unzipDir)
        self._removeOldUnSignedZip(zipName=zipName)
        self._recreateSignedZip(unzipDir=unzipDir, zipName=zipName)
        self._moveSignedZipBack(originalZipDir=originalZipDir, zipName=zipName)

    def _cleanupTemporaryDirectory(self, unzipDir: str, zipName: str):
        # rm - rf "${TEMP_DIR}/${ZIP_NAME}"
        # rm - rf ${PYTHON_UNZIP_DIR}

        deleteUnzipDir: str = f'rm -rf {unzipDir}'
        self._runCommand(deleteUnzipDir)
        deleteZipCopy: str = f'rm -rf {TMP_DIR_PATH}{osSep}{zipName}'
        self._runCommand(deleteZipCopy)

    def _getUnsignedZipCopy(self, originalZipDir: str, zipName: str):
        # echo "Get copy of unsigned zip file"
        # cp - vp ${PYTHON_ZIP} ${TEMP_DIR}
        # export PYTHON_ZIP="${ORIGINAL_ZIP_DIR}/${ZIP_NAME}"
        # export PYTHON_UNZIP_DIR="${TEMP_DIR}/${UNZIP_DIR}"
        pythonZip: str = f'{originalZipDir}/{zipName}'

        options:  str = self._getToolOptions(verboseOptions=COPY_OPTIONS_VERBOSE, quietOptions=COPY_OPTIONS_QUIET)
        makeCopy: str = f'cp {options} {pythonZip} {TMP_DIR_PATH}'

        self._runCommand(makeCopy)

    def _unzipCopy(self, unzipDir: str, zipName: str):
        # echo "Unzip it"
        # /usr/bin/ditto -v -x -k "${TEMP_DIR}/${ZIP_NAME}" "${TEMP_DIR}/${UNZIP_DIR}"
        options: str = self._getToolOptions(verboseOptions=DITTO_EXTRACT_OPTIONS_VERBOSE, quietOptions=DITTO_EXTRACT_OPTIONS_QUIET)
        unZipIt: str = f'{DITTO_TOOL} {options} {TMP_DIR_PATH}/{zipName} {unzipDir}'

        self._runCommand(unZipIt)

    def _signLibs(self, unzipDir: str):
        # noinspection SpellCheckingInspection
        """
        export  OPTIONS = "--force --verbose --timestamp --options=runtime "
        find "${PYTHON_UNZIP_DIR}/PIL/.dylibs" -iname '*.dylib' |

        Args:
            unzipDir:
        """
        # noinspection SpellCheckingInspection
        p:        Path = Path(f'{unzipDir}/PIL/.dylibs')
        if p.exists() is False:
            secho(f'No dynamic libraries to sign at: {p}')
        else:
            identity: str  = self._extendedEnvironment.identity
            options:  str  = self._getToolOptions(verboseOptions=CODE_SIGN_OPTIONS_VERBOSE, quietOptions=CODE_SIGN_OPTIONS_QUIET)
            if self.verbose is True:
                self._zipSignVerbose(identity, options, p)
            else:
                self._zipSignProgressBar(identity, options, p)

    def _removeOldUnSignedZip(self, zipName: str):
        options:    str  = self._getToolOptions(verboseOptions=REMOVE_OPTIONS_VERBOSE, quietOptions=REMOVE_OPTIONS_QUIET)
        path:       Path = Path(TMP_DIR_PATH) / zipName
        removeCopy: str  = f'rm {options} {path}'

        self._runCommand(removeCopy)

    def _recreateSignedZip(self, unzipDir: str, zipName: str):
        options:    str = self._getToolOptions(verboseOptions=DITTO_CREATE_OPTIONS_VERBOSE, quietOptions=DITTO_CREATE_OPTIONS_QUIET)
        tmpDirPath: Path = Path(TMP_DIR_PATH) / zipName
        zipIt:   str = f'{DITTO_TOOL} {options} {unzipDir} {tmpDirPath}'

        self._runCommand(zipIt)

    def _moveSignedZipBack(self, originalZipDir: str, zipName: str):

        signedZip: Path = Path(TMP_DIR_PATH) / zipName

        options:   str  = self._getToolOptions(verboseOptions=COPY_OPTIONS_VERBOSE, quietOptions=COPY_OPTIONS_QUIET)

        moveItBack: str = f'cp {options} {signedZip} {originalZipDir}'

        self._runCommand(moveItBack)

    def _zipSignVerbose(self, identity: str, options: str, dynamicLibrariesPath: Path):

        for lib in dynamicLibrariesPath.iterdir():
            signIt: str = f'{CODE_SIGN_TOOL} --sign "{identity}" {options} {lib}'
            self._runCommand(signIt)

    def _zipSignProgressBar(self, identity: str, options: str, dynamicLibrariesPath: Path):

        libs = list(dynamicLibrariesPath.iterdir())
        pbar: tqdm = tqdm(libs, unit=PROGRESS_BAR_UNITS, bar_format='{l_bar}{bar}{r_bar}')
        for lib in pbar:
            path: Path = Path(lib)
            pbar.set_description(f'Processing {path.name}', refresh=True)
            signIt: str = f'{CODE_SIGN_TOOL} --sign "{identity}" {options} {lib}'
            self._runCommand(signIt)



from abc import ABCMeta
from logging import Logger
from logging import getLogger

from sys import stdout

from subprocess import run as subProcessRun
from subprocess import check_call as subProcessCheckCall

from subprocess import STDOUT

from abc import abstractmethod

from click import secho

from py2appsigner.BaseCommand import BaseCommand
from py2appsigner.environment.BasicEnvironment import BasicEnvironment

BUILD_DIR:  str = '/dist/'
ZIP_SUFFIX: str = 'zip'

CODE_SIGN_TOOL:            str = '/usr/bin/codesign'
DITTO_TOOL:                str = '/usr/bin/ditto'

COPY_OPTIONS_VERBOSE:          str = '-vp'
COPY_OPTIONS_QUIET:            str = '-p'
REMOVE_OPTIONS_VERBOSE:        str = '-vrf'
REMOVE_OPTIONS_QUIET:          str = '-rf'
CODE_SIGN_OPTIONS_VERBOSE:     str = '-vvvv --force --timestamp --options=runtime'
CODE_SIGN_OPTIONS_QUIET:       str = '--force --timestamp --options=runtime'


class MetaBaseCommand(ABCMeta, type(BaseCommand)):      # type: ignore
    """
    I have know idea why this works:
    https://stackoverflow.com/questions/66591752/metaclass-conflict-when-trying-to-create-a-python-abstract-class-that-also-subcl
    """
    pass


class CommandBasic(BaseCommand):
    __metaclass = MetaBaseCommand

    def __init__(self, environment: BasicEnvironment):

        super().__init__(verbose=environment.verbose)
        self._basicEnvironment: BasicEnvironment = environment

        self.baseLogger: Logger = getLogger(__name__)

        self._fullPath:        str = f'{self._basicEnvironment.projectsBase}/{self._basicEnvironment.projectDirectory}'
        self._applicationName: str = f'{self._fullPath}{BUILD_DIR}{self._basicEnvironment.applicationName}.app'

    @abstractmethod
    def execute(self):
        pass
